"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
var _this = this;
Object.defineProperty(exports, "__esModule", { value: true });
var loginService_1 = require("../../services/loginService");
var mockFactory_1 = require("../../test/mockFactory");
var utils_1 = require("../../test/utils");
var azureLoginPlugin_1 = require("./azureLoginPlugin");
describe("Login Plugin", function () {
    var authResponse = mockFactory_1.MockFactory.createTestAuthResponse();
    var envVariables = mockFactory_1.MockFactory.createTestServicePrincipalEnvVariables();
    var credentials = mockFactory_1.MockFactory.createTestVariables().azureCredentials;
    function createPlugin(hasCreds, serverless, options) {
        if (hasCreds === void 0) { hasCreds = false; }
        var sls = serverless || mockFactory_1.MockFactory.createTestServerless();
        if (!hasCreds) {
            delete sls.variables["azureCredentials"];
        }
        return new azureLoginPlugin_1.AzureLoginPlugin(sls, options || mockFactory_1.MockFactory.createTestServerlessOptions());
    }
    function createMockLoginFunction() {
        return jest.fn(function () { return Promise.resolve(authResponse); });
    }
    function setServicePrincipalEnvVariables() {
        utils_1.setEnvVariables(envVariables);
    }
    function unsetServicePrincipalEnvVariables() {
        utils_1.unsetEnvVariables(envVariables);
    }
    function invokeLoginHook(hasCreds, serverless, options) {
        if (hasCreds === void 0) { hasCreds = false; }
        return __awaiter(this, void 0, void 0, function () {
            var plugin;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        plugin = createPlugin(hasCreds, serverless, options);
                        return [4 /*yield*/, utils_1.invokeHook(plugin, "before:deploy:deploy")];
                    case 1:
                        _a.sent();
                        return [2 /*return*/];
                }
            });
        });
    }
    beforeEach(function () {
        loginService_1.AzureLoginService.interactiveLogin = createMockLoginFunction();
        loginService_1.AzureLoginService.servicePrincipalLogin = createMockLoginFunction();
    });
    it("returns if azure credentials are set", function () { return __awaiter(_this, void 0, void 0, function () {
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0: return [4 /*yield*/, invokeLoginHook(true)];
                case 1:
                    _a.sent();
                    expect(loginService_1.AzureLoginService.interactiveLogin).not.toBeCalled();
                    expect(loginService_1.AzureLoginService.servicePrincipalLogin).not.toBeCalled();
                    return [2 /*return*/];
            }
        });
    }); });
    it("calls login if azure credentials are not set", function () { return __awaiter(_this, void 0, void 0, function () {
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0: return [4 /*yield*/, invokeLoginHook()];
                case 1:
                    _a.sent();
                    expect(loginService_1.AzureLoginService.interactiveLogin).toBeCalled();
                    expect(loginService_1.AzureLoginService.servicePrincipalLogin).not.toBeCalled();
                    return [2 /*return*/];
            }
        });
    }); });
    it("calls service principal login if environment variables are set", function () { return __awaiter(_this, void 0, void 0, function () {
        var sls;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    setServicePrincipalEnvVariables();
                    sls = mockFactory_1.MockFactory.createTestServerless();
                    return [4 /*yield*/, invokeLoginHook(false, sls)];
                case 1:
                    _a.sent();
                    expect(loginService_1.AzureLoginService.servicePrincipalLogin).toBeCalledWith("azureServicePrincipalClientId", "azureServicePrincipalPassword", "azureServicePrincipalTenantId", undefined // would be options
                    );
                    expect(loginService_1.AzureLoginService.interactiveLogin).not.toBeCalled();
                    expect(sls.variables["azureCredentials"]).toEqual(credentials);
                    expect(sls.variables["subscriptionId"]).toEqual("azureSubId");
                    return [2 /*return*/];
            }
        });
    }); });
    it("calls interactive login if environment variables are not set", function () { return __awaiter(_this, void 0, void 0, function () {
        var sls;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    unsetServicePrincipalEnvVariables();
                    sls = mockFactory_1.MockFactory.createTestServerless();
                    return [4 /*yield*/, invokeLoginHook(false, sls)];
                case 1:
                    _a.sent();
                    expect(loginService_1.AzureLoginService.servicePrincipalLogin).not.toBeCalled();
                    expect(loginService_1.AzureLoginService.interactiveLogin).toBeCalled();
                    expect(sls.variables["azureCredentials"]).toEqual(credentials);
                    expect(sls.variables["subscriptionId"]).toEqual("azureSubId");
                    return [2 /*return*/];
            }
        });
    }); });
    it("logs an error from authentication and exits process", function () { return __awaiter(_this, void 0, void 0, function () {
        var errorMessage, sls;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    unsetServicePrincipalEnvVariables();
                    process.exit = jest.fn();
                    errorMessage = "This is my error message";
                    loginService_1.AzureLoginService.interactiveLogin = jest.fn(function () {
                        throw new Error(errorMessage);
                    });
                    sls = mockFactory_1.MockFactory.createTestServerless();
                    return [4 /*yield*/, invokeLoginHook(false, sls)];
                case 1:
                    _a.sent();
                    expect(loginService_1.AzureLoginService.interactiveLogin).toBeCalled();
                    expect(loginService_1.AzureLoginService.servicePrincipalLogin).not.toBeCalled();
                    expect(sls.cli.log).lastCalledWith("Error: " + errorMessage);
                    expect(process.exit).toBeCalledWith(0);
                    return [2 /*return*/];
            }
        });
    }); });
    it("Uses the user specified subscription ID", function () { return __awaiter(_this, void 0, void 0, function () {
        var sls, opt;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    sls = mockFactory_1.MockFactory.createTestServerless();
                    opt = mockFactory_1.MockFactory.createTestServerlessOptions();
                    opt["subscriptionId"] = "test-subs-id";
                    return [4 /*yield*/, invokeLoginHook(false, sls, opt)];
                case 1:
                    _a.sent();
                    expect(loginService_1.AzureLoginService.interactiveLogin).toBeCalled();
                    expect(sls.variables["subscriptionId"]).toEqual("test-subs-id");
                    expect(sls.cli.log).toBeCalledWith("Using subscription ID: test-subs-id");
                    return [2 /*return*/];
            }
        });
    }); });
    it("Uses the default subscription ID", function () { return __awaiter(_this, void 0, void 0, function () {
        var sls, opt;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    sls = mockFactory_1.MockFactory.createTestServerless();
                    opt = mockFactory_1.MockFactory.createTestServerlessOptions();
                    return [4 /*yield*/, invokeLoginHook(false, sls, opt)];
                case 1:
                    _a.sent();
                    expect(loginService_1.AzureLoginService.interactiveLogin).toBeCalled();
                    expect(sls.variables["subscriptionId"]).toEqual("azureSubId");
                    expect(sls.cli.log).toBeCalledWith("Using subscription ID: azureSubId");
                    return [2 /*return*/];
            }
        });
    }); });
});
