"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var _this = this;
Object.defineProperty(exports, "__esModule", { value: true });
var fs_1 = __importDefault(require("fs"));
var mock_fs_1 = __importDefault(require("mock-fs"));
var rimraf_1 = __importDefault(require("rimraf"));
var mockFactory_1 = require("../test/mockFactory");
var funcService_1 = require("./funcService");
describe("Azure Func Service", function () {
    function createService(sls, options) {
        return new funcService_1.FuncService(sls || mockFactory_1.MockFactory.createTestServerless(), options || mockFactory_1.MockFactory.createTestServerlessOptions());
    }
    describe("Add command", function () {
        beforeAll(function () {
            mock_fs_1.default({
                "myExistingFunction": {
                    "index.js": "contents",
                    "function.json": "contents",
                },
                "serverless.yml": mockFactory_1.MockFactory.createTestServerlessYml(true),
            });
        });
        afterAll(function () {
            mock_fs_1.default.restore();
        });
        afterEach(function () {
            jest.clearAllMocks();
        });
        it("returns with missing name", function () { return __awaiter(_this, void 0, void 0, function () {
            var sls, options, service;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        sls = mockFactory_1.MockFactory.createTestServerless();
                        options = mockFactory_1.MockFactory.createTestServerlessOptions();
                        service = createService(sls, options);
                        return [4 /*yield*/, service.add()];
                    case 1:
                        _a.sent();
                        expect(sls.cli.log).toBeCalledWith("Need to provide a name of function to add");
                        return [2 /*return*/];
                }
            });
        }); });
        it("returns with pre-existing function", function () { return __awaiter(_this, void 0, void 0, function () {
            var sls, options, service;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        sls = mockFactory_1.MockFactory.createTestServerless();
                        options = mockFactory_1.MockFactory.createTestServerlessOptions();
                        options["name"] = "hello";
                        service = createService(sls, options);
                        return [4 /*yield*/, service.add()];
                    case 1:
                        _a.sent();
                        expect(sls.cli.log).toBeCalledWith("Function hello already exists");
                        return [2 /*return*/];
                }
            });
        }); });
        it("creates function handler and updates serverless.yml", function () { return __awaiter(_this, void 0, void 0, function () {
            var sls, options, functionName, expectedFunctionsYml, service, writeFileCalls;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        sls = mockFactory_1.MockFactory.createTestServerless();
                        options = mockFactory_1.MockFactory.createTestServerlessOptions();
                        functionName = "myFunction";
                        options["name"] = functionName;
                        expectedFunctionsYml = mockFactory_1.MockFactory.createTestSlsFunctionConfig();
                        expectedFunctionsYml[functionName] = mockFactory_1.MockFactory.createTestFunctionMetadata(functionName);
                        service = createService(sls, options);
                        return [4 /*yield*/, service.add()];
                    case 1:
                        _a.sent();
                        writeFileCalls = sls.utils.writeFileSync.mock.calls;
                        expect(writeFileCalls[0][0]).toBe("./" + functionName + ".js");
                        expect(writeFileCalls[1][0]).toBe("serverless.yml");
                        expect(writeFileCalls[1][1]).toBe(mockFactory_1.MockFactory.createTestServerlessYml(true, expectedFunctionsYml));
                        return [2 /*return*/];
                }
            });
        }); });
    });
    describe("Remove command", function () {
        beforeAll(function () {
            mock_fs_1.default({
                "index.js": "contents",
                "function1": {
                    "function.json": "contents",
                },
            });
        });
        afterAll(function () {
            mock_fs_1.default.restore();
        });
        it("returns with missing name", function () { return __awaiter(_this, void 0, void 0, function () {
            var sls, options, service;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        sls = mockFactory_1.MockFactory.createTestServerless();
                        options = mockFactory_1.MockFactory.createTestServerlessOptions();
                        service = createService(sls, options);
                        return [4 /*yield*/, service.remove()];
                    case 1:
                        _a.sent();
                        expect(sls.cli.log).toBeCalledWith("Need to provide a name of function to remove");
                        return [2 /*return*/];
                }
            });
        }); });
        it("returns with non-existing function", function () { return __awaiter(_this, void 0, void 0, function () {
            var sls, options, service;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        sls = mockFactory_1.MockFactory.createTestServerless();
                        options = mockFactory_1.MockFactory.createTestServerlessOptions();
                        options["name"] = "myNonExistingFunction";
                        service = createService(sls, options);
                        return [4 /*yield*/, service.remove()];
                    case 1:
                        _a.sent();
                        expect(sls.cli.log).toBeCalledWith("Function myNonExistingFunction does not exist");
                        return [2 /*return*/];
                }
            });
        }); });
        it("deletes directory and updates serverless.yml", function () { return __awaiter(_this, void 0, void 0, function () {
            var sls, options, functionName, unlinkSpy, rimrafSpy, service, expectedFunctionsYml;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        mock_fs_1.default({
                            "hello.js": "contents",
                            hello: {
                                "function.json": "contents",
                            }
                        });
                        sls = mockFactory_1.MockFactory.createTestServerless();
                        options = mockFactory_1.MockFactory.createTestServerlessOptions();
                        functionName = "hello";
                        options["name"] = functionName;
                        unlinkSpy = jest.spyOn(fs_1.default, "unlinkSync");
                        rimrafSpy = jest.spyOn(rimraf_1.default, "sync");
                        service = createService(sls, options);
                        return [4 /*yield*/, service.remove()];
                    case 1:
                        _a.sent();
                        expect(unlinkSpy).toBeCalledWith(functionName + ".js");
                        expect(rimrafSpy).toBeCalledWith(functionName);
                        unlinkSpy.mockRestore();
                        rimrafSpy.mockRestore();
                        expectedFunctionsYml = mockFactory_1.MockFactory.createTestSlsFunctionConfig();
                        delete expectedFunctionsYml[functionName];
                        expect(sls.utils.writeFileSync).toBeCalledWith("serverless.yml", mockFactory_1.MockFactory.createTestServerlessYml(true, expectedFunctionsYml));
                        return [2 /*return*/];
                }
            });
        }); });
        it("does not try to delete file or directory if they don't exist", function () { return __awaiter(_this, void 0, void 0, function () {
            var sls, options, functionName, unlinkSpy, rimrafSpy, service, expectedFunctionsYml;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        mock_fs_1.default({});
                        sls = mockFactory_1.MockFactory.createTestServerless();
                        options = mockFactory_1.MockFactory.createTestServerlessOptions();
                        functionName = "hello";
                        options["name"] = functionName;
                        unlinkSpy = jest.spyOn(fs_1.default, "unlinkSync");
                        rimrafSpy = jest.spyOn(rimraf_1.default, "sync");
                        service = createService(sls, options);
                        return [4 /*yield*/, service.remove()];
                    case 1:
                        _a.sent();
                        expect(unlinkSpy).not.toBeCalled();
                        expect(rimrafSpy).not.toBeCalled();
                        unlinkSpy.mockRestore();
                        rimrafSpy.mockRestore();
                        expectedFunctionsYml = mockFactory_1.MockFactory.createTestSlsFunctionConfig();
                        delete expectedFunctionsYml[functionName];
                        expect(sls.utils.writeFileSync).toBeCalledWith("serverless.yml", mockFactory_1.MockFactory.createTestServerlessYml(true, expectedFunctionsYml));
                        return [2 /*return*/];
                }
            });
        }); });
    });
});
