"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var _this = this;
Object.defineProperty(exports, "__esModule", { value: true });
var fs_1 = __importDefault(require("fs"));
var mock_fs_1 = __importDefault(require("mock-fs"));
var path_1 = __importDefault(require("path"));
var mockFactory_1 = require("../test/mockFactory");
var offlineService_1 = require("./offlineService");
describe("Offline Service", function () {
    function createService(sls) {
        return new offlineService_1.OfflineService(sls || mockFactory_1.MockFactory.createTestServerless(), mockFactory_1.MockFactory.createTestServerlessOptions());
    }
    beforeEach(function () {
        // Mocking the file system so that files are not created in project directory
        mock_fs_1.default({});
    });
    afterEach(function () {
        mock_fs_1.default.restore();
    });
    it("builds required files for offline execution", function () { return __awaiter(_this, void 0, void 0, function () {
        var sls, service, writeFileSpy, calls, functionNames, i, name_1;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    sls = mockFactory_1.MockFactory.createTestServerless();
                    service = createService(sls);
                    writeFileSpy = jest.spyOn(fs_1.default, "writeFileSync");
                    return [4 /*yield*/, service.build()];
                case 1:
                    _a.sent();
                    calls = writeFileSpy.mock.calls;
                    functionNames = sls.service.getAllFunctions();
                    expect(calls).toHaveLength(functionNames.length + 1);
                    for (i = 0; i < functionNames.length; i++) {
                        name_1 = functionNames[i];
                        expect(calls[i][0]).toEqual("" + name_1 + path_1.default.sep + "function.json");
                        expect(JSON.parse(calls[i][1])).toEqual(mockFactory_1.MockFactory.createTestBindingsObject(".." + path_1.default.sep + name_1 + ".js"));
                    }
                    expect(calls[calls.length - 1][0]).toEqual("local.settings.json");
                    writeFileSpy.mockRestore();
                    return [2 /*return*/];
            }
        });
    }); });
    it("does not write files if they already exist", function () { return __awaiter(_this, void 0, void 0, function () {
        var sls, service, functionNames, writeFileSpy, calls;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    sls = mockFactory_1.MockFactory.createTestServerless();
                    service = createService(sls);
                    functionNames = sls.service.getAllFunctions();
                    writeFileSpy = jest.spyOn(fs_1.default, "writeFileSync");
                    mock_fs_1.default({
                        "local.settings.json": "contents",
                    });
                    return [4 /*yield*/, service.build()];
                case 1:
                    _a.sent();
                    calls = writeFileSpy.mock.calls;
                    expect(calls).toHaveLength(functionNames.length);
                    return [2 /*return*/];
            }
        });
    }); });
    it("cleans up functions files", function () { return __awaiter(_this, void 0, void 0, function () {
        var sls, service, unlinkSpy, rmdirSpy, unlinkCalls, rmdirCalls;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    mock_fs_1.default({
                        hello: {
                            "function.json": "contents"
                        },
                        goodbye: {
                            "function.json": "contents"
                        },
                        "local.settings.json": "contents",
                    });
                    sls = mockFactory_1.MockFactory.createTestServerless();
                    service = createService(sls);
                    unlinkSpy = jest.spyOn(fs_1.default, "unlinkSync");
                    rmdirSpy = jest.spyOn(fs_1.default, "rmdirSync");
                    return [4 /*yield*/, service.cleanup()];
                case 1:
                    _a.sent();
                    unlinkCalls = unlinkSpy.mock.calls;
                    expect(unlinkCalls[0][0]).toBe("hello" + path_1.default.sep + "function.json");
                    expect(unlinkCalls[1][0]).toBe("goodbye" + path_1.default.sep + "function.json");
                    expect(unlinkCalls[2][0]).toBe("local.settings.json");
                    rmdirCalls = rmdirSpy.mock.calls;
                    expect(rmdirCalls[0][0]).toBe("hello");
                    expect(rmdirCalls[1][0]).toBe("goodbye");
                    unlinkSpy.mockRestore();
                    rmdirSpy.mockRestore();
                    return [2 /*return*/];
            }
        });
    }); });
    it("does not try to remove files if they don't exist", function () { return __awaiter(_this, void 0, void 0, function () {
        var sls, service, unlinkSpy, rmdirSpy, unlinkCalls, rmdirCalls;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    mock_fs_1.default({
                        hello: {
                            "function.json": "contents"
                        },
                        goodbye: {
                            "function.json": "contents"
                        },
                    });
                    sls = mockFactory_1.MockFactory.createTestServerless();
                    service = createService(sls);
                    unlinkSpy = jest.spyOn(fs_1.default, "unlinkSync");
                    rmdirSpy = jest.spyOn(fs_1.default, "rmdirSync");
                    return [4 /*yield*/, service.cleanup()];
                case 1:
                    _a.sent();
                    unlinkCalls = unlinkSpy.mock.calls;
                    expect(unlinkCalls).toHaveLength(2);
                    expect(unlinkCalls[0][0]).toBe("hello" + path_1.default.sep + "function.json");
                    expect(unlinkCalls[1][0]).toBe("goodbye" + path_1.default.sep + "function.json");
                    rmdirCalls = rmdirSpy.mock.calls;
                    expect(rmdirCalls[0][0]).toBe("hello");
                    expect(rmdirCalls[1][0]).toBe("goodbye");
                    unlinkSpy.mockRestore();
                    rmdirSpy.mockRestore();
                    return [2 /*return*/];
            }
        });
    }); });
    it("instructs users how to run locally", function () { return __awaiter(_this, void 0, void 0, function () {
        var sls, service;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    sls = mockFactory_1.MockFactory.createTestServerless();
                    service = createService(sls);
                    return [4 /*yield*/, service.start()];
                case 1:
                    _a.sent();
                    // Trivial test for now. In the future, this process
                    // may spawn the start process itself rather than telling
                    // the user how to do it.
                    expect(sls.cli.log).toBeCalledTimes(3);
                    return [2 /*return*/];
            }
        });
    }); });
});
