import Serverless from "serverless";
export interface FunctionMetadata {
    entryPoint: any;
    handlerPath: any;
    params: any;
}
export declare class Utils {
    static getFunctionMetaData(functionName: string, serverless: Serverless): FunctionMetadata;
    static getEntryPointAndHandlerPath(handler: string): {
        entryPoint: string;
        handlerPath: string;
    };
    static prettyPrint(json: any): string;
    /**
     * Take the first `substringSize` characters from each string and return as one string
     * @param substringSize Size of substring to take from beginning of each string
     * @param args Strings to take substrings from
     */
    static appendSubstrings(substringSize: number, ...args: string[]): string;
    /**
     * Creates a short name to be used for state name abbreviation
     * @param stageName The stage name
     */
    static createShortStageName(stageName: string): string;
    /**
     * Gets the normalized region name from long name (ex. West US 2 -> westus2)
     * @param regionName The region name
     */
    static getNormalizedRegionName(regionName: string): string;
    /**
     * Creates a short name for an azure region
     * @param regionName The azure region name
     */
    static createShortAzureRegionName(regionName: string): string;
    /**
     * Creates a short name from a long name based on a well-known string map
     * @param longName The long name to replace
     * @param wellKnownMap A well known map of long terms to short abbreviations
     */
    private static createShortName;
    static get(object: any, key: string, defaultValue?: any): any;
    static getTimestampFromName(name: string): string;
}
